@@ -4,13 +4,14 @@ use 5.008;
 
 use Moose::Util::TypeConstraints;
 use MooseX::Meta::TypeConstraint::Structured;
+use MooseX::Meta::TypeConstraint::Structured::Optional;
 use MooseX::Types::Structured::OverflowHandler;
-use MooseX::Types -declare => [qw(Dict Tuple Optional)];
-use Sub::Exporter -setup => { exports => [ qw(Dict Tuple Optional slurpy) ] };
+use MooseX::Types -declare => [qw(Dict Map Tuple Optional)];
+use Sub::Exporter -setup => [ qw(Dict Map Tuple Optional slurpy) ];
 use Devel::PartialDump;
 use Scalar::Util qw(blessed);
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 our $AUTHORITY = 'cpan:JJNAPIORK';
 
 =head1 NAME
@@ -22,7 +23,7 @@ MooseX::Types::Structured - Structured Type Constraints for Moose
 The following is example usage for this module.
 
     package Person;
-	
+
     use Moose;
     use MooseX::Types::Moose qw(Str Int HashRef);
     use MooseX::Types::Structured qw(Dict Tuple Optional);
@@ -35,7 +36,7 @@ The following is example usage for this module.
             middle => Optional[Str],
         ],
     );
-    
+
     ## description is a string field followed by a HashRef of tagged data.
     has description => (
       isa=>Tuple[
@@ -76,30 +77,30 @@ But all of these would cause a constraint error for the 'name' attribute:
 
     ## Value for 'name' not a HashRef
     Person->new( name => 'John' );
-    
+
     ## Value for 'name' has incorrect hash key and missing required keys
     Person->new( name => {
         first_name => 'John'
     });
-    
+
     ## Also incorrect keys
     Person->new( name => {
         first_name => 'John',
         age => 39,
     });
-    
+
     ## key 'middle' incorrect type, should be a Str not a ArrayRef
     Person->new( name => {
         first => 'Vanessa',
         middle => [1,2],
         last => 'Li',
-    }); 
+    });
 
 And these would cause a constraint error for the 'description' attribute:
 
     ## Should be an ArrayRef
     Person->new( description => 'Hello I am a String' );
-    
+
     ## First element must be a string not a HashRef.
     Person->new (description => [{
         tag1 => 'value1',
@@ -117,7 +118,7 @@ generalized form is:
 
     TypeConstraint[@TypeParameters or %TypeParameters]
 
-Where 'TypeParameters' is an array reference or hash references of 
+Where 'TypeParameters' is an array reference or hash references of
 L<Moose::Meta::TypeConstraint> objects.
 
 This type library enables structured type constraints. It is built on top of the
@@ -132,7 +133,7 @@ have similar functionality, so their syntax is likewise similar. For example,
 you could define a parameterized constraint like:
 
     subtype ArrayOfInts,
-     as Arrayref[Int];
+     as ArrayRef[Int];
 
 which would constrain a value to something like [1,2,3,...] and so on.  On the
 other hand, a structured type constraint explicitly names all it's allowed
@@ -140,7 +141,7 @@ other hand, a structured type constraint explicitly names all it's allowed
 
     subtype StringFollowedByInt,
      as Tuple[Str,Int];
-	
+
 would constrain it's value to things like ['hello', 111] but ['hello', 'world']
 would fail, as well as ['hello', 111, 'world'] and so on.  Here's another
 example:
@@ -156,20 +157,20 @@ example:
         Str, Int,
         Optional[HashRef]
      ];
-     
+
 This defines a type constraint that validates values like:
 
     ['Hello', 100, {key1 => 'value1', key2 => 'value2'}];
     ['World', 200];
-    
+
 Notice that the last type constraint in the structure is optional.  This is
 enabled via the helper Optional type constraint, which is a variation of the
 core Moose type constraint 'Maybe'.  The main difference is that Optional type
-constraints are required to validate if they exist, while 'Maybe' permits 
+constraints are required to validate if they exist, while 'Maybe' permits
 undefined values.  So the following example would not validate:
 
     StringIntOptionalHashRef->validate(['Hello Undefined', 1000, undef]);
-    
+
 Please note the subtle difference between undefined and null.  If you wish to
 allow both null and undefined, you should use the core Moose 'Maybe' type
 constraint instead:
@@ -188,7 +189,7 @@ constraint instead:
 This would validate the following:
 
     ['Hello', 100, {key1 => 'value1', key2 => 'value2'}];
-    ['World', 200, undef];    
+    ['World', 200, undef];
     ['World', 200];
 
 Structured constraints are not limited to arrays.  You can define a structure
@@ -203,17 +204,17 @@ against a HashRef with the 'Dict' type constaint as in this example:
 This would constrain a HashRef that validates something like:
 
     {firstname => 'Christopher', lastname => 'Parsons'};
-    
+
 but all the following would fail validation:
 
     ## Incorrect keys
     {first => 'Christopher', last => 'Parsons'};
-    
+
     ## Too many keys
     {firstname => 'Christopher', lastname => 'Parsons', middlename => 'Allen'};
-    
+
     ## Not a HashRef
-    ['Christopher', 'Parsons']; 
+    ['Christopher', 'Parsons'];
 
 These structures can be as simple or elaborate as you wish.  You can even
 combine various structured, parameterized and simple constraints all together:
@@ -224,13 +225,13 @@ combine various structured, parameterized and simple constraints all together:
         Dict[name=>Str, age=>Int],
         ArrayRef[Int]
      ];
-	
+
 Which would match:
 
 	[1, {name=>'John', age=>25},[10,11,12]];
 
 Please notice how the type parameters can be visually arranged to your liking
-and to improve the clarity of your meaning.  You don't need to run then 
+and to improve the clarity of your meaning.  You don't need to run then
 altogether onto a single line.  Additionally, since the 'Dict' type constraint
 defines a hash constraint, the key order is not meaningful.  For example:
 
@@ -256,34 +257,34 @@ example:
 
     package MyApp::MyStruct;
     use Moose;
-    
+
     ## lazy way to make a bunch of attributes
     has $_ for qw(full_name age_in_years);
-    
+
     package MyApp::MyClass;
     use Moose;
-    
-    has person => (isa => 'MyApp::MyStruct');		
-    
+
+    has person => (isa => 'MyApp::MyStruct');
+
     my $instance = MyApp::MyClass->new(
         person=>MyApp::MyStruct->new(
             full_name => 'John',
             age_in_years => 39,
         ),
     );
-	
+
 This method may take some additional time to setup but will give you more
 flexibility.  However, structured constraints are highly compatible with this
 method, granting some interesting possibilities for coercion.  Try:
 
     package MyApp::MyClass;
-    
+
     use Moose;
     use MyApp::MyStruct;
-    
+
     ## It's recommended your type declarations live in a separate class in order
     ## to promote reusability and clarity.  Inlined here for brevity.
-    
+
     use MooseX::Types::DateTime qw(DateTime);
     use MooseX::Types -declare [qw(MyStruct)];
     use MooseX::Types::Moose qw(Str Int);
@@ -296,7 +297,7 @@ method, granting some interesting possibilities for coercion.  Try:
     ## Just a shorter version really.
     subtype MyStruct,
      as 'MyApp::MyStruct';
-    
+
     ## Add the coercions.
     coerce MyStruct,
      from Dict[
@@ -312,22 +313,22 @@ method, granting some interesting possibilities for coercion.  Try:
      ], via {
         my $name = $_->{firstname} .' '. $_->{lastname};
         my $age = DateTime->now - $_->{dob};
-        
+
         MyApp::MyStruct->new(
             full_name=>$name,
             age_in_years=>$age->years,
         );
      };
-     
-    has person => (isa=>MyStruct);	
-     
+
+    has person => (isa=>MyStruct);
+
 This would allow you to instantiate with something like:
 
     my $obj = MyApp::MyClass->new( person => {
         full_name=>'John Napiorkowski',
         age_in_years=>39,
     });
-    
+
 Or even:
 
     my $obj = MyApp::MyClass->new( person => {
@@ -347,13 +348,13 @@ this example:
 
     subtype Person,
      as Dict[name => Str];
-	 
+
     subtype FriendlyPerson,
      as Person[
         name => Str,
         total_friends => Int,
      ];
-	 
+
 This will actually work BUT you have to take care that the subtype has a
 structure that does not contradict the structure of it's parent.  For now the
 above works, but I will clarify the syntax for this at a future point, so
@@ -371,13 +372,13 @@ Coercions currently work for 'one level' deep.  That is you can do:
         name => Str,
         age => Int
     ];
-    
+
     subtype Fullname,
      as Dict[
         first => Str,
         last => Str
      ];
-    
+
     coerce Person,
      ## Coerce an object of a particular class
      from BlessedPersonObject, via {
@@ -386,7 +387,7 @@ Coercions currently work for 'one level' deep.  That is you can do:
             age=>$_->age,
         };
      },
-     
+
      ## Coerce from [$name, $age]
      from ArrayRef, via {
         +{
@@ -404,7 +405,7 @@ Coercions currently work for 'one level' deep.  That is you can do:
             age =>$age->years
         }
      };
-	 
+
 And that should just work as expected.  However, if there are any 'inner'
 coercions, such as a coercion on 'Fullname' or on 'DateTime', that coercion
 won't currently get activated.
@@ -426,7 +427,7 @@ example:
 	 		ArrayRef[Person]
 	 	],
 	 ];
-	 
+
 This would declare a Person subtype that contains a name and an optional
 ArrayRef of Persons who are friends as in:
 
@@ -453,7 +454,7 @@ a Union with an non recursive option such as:
 	 	Str,
 	 	Str|Tuple,
 	 ];
-	 
+
 Which validates:
 
 	[
@@ -467,7 +468,7 @@ Which validates:
 		],
 	];
 
-Otherwise you will define a subtype thatis impossible to validate since it is 
+Otherwise you will define a subtype thatis impossible to validate since it is
 infinitely recursive.  For more information about defining recursive types,
 please see the documentation in L<MooseX::Types> and the test cases.
 
@@ -486,7 +487,7 @@ list of contained constraints.  For example:
 The Values of @constraints should ideally be L<MooseX::Types> declared type
 constraints.  We do support 'old style' L<Moose> string based constraints to a
 limited degree but these string type constraints are considered deprecated.
-There will be limited support for bugs resulting from mixing string and 
+There will be limited support for bugs resulting from mixing string and
 L<MooseX::Types> in your structures.  If you encounter such a bug and really
 need it fixed, we will required a detailed test case at the minimum.
 
@@ -497,9 +498,20 @@ hashref.  For example:
 
     Dict[name=>Str, age=>Int]; ## Validates {name=>'John', age=>39}
 
-The keys in %constraints follow the same rules as @constraints in the above 
+The keys in %constraints follow the same rules as @constraints in the above
 section.
 
+=head2 Map[ $key_constraint, $value_constraint ]
+
+This defines a HashRef based constraint in which both the keys and values are
+required to meet certain constraints.  For example, to map hostnames to IP
+addresses, you might say:
+
+  Map[ HostName, IPAddress ]
+
+The type constraint would only be met if every key was a valid HostName and
+every value was a valid IPAddress.
+
 =head2 Optional[$constraint]
 
 This is primarily a helper constraint for Dict and Tuple type constraints.  What
@@ -514,7 +526,7 @@ or a tuple where some of the values are not required.  For example:
         last=>Str,
         middle=>Optional[Str],
     ];
-        
+
 Creates a constraint that validates against a hashref with the keys 'first' and
 'last' being strings and required while an optional key 'middle' is must be a
 string if it appears but doesn't have to appear.  So in this case both the
@@ -556,7 +568,7 @@ In order to allow structured validation of, "and then some", arguments, you can
 use the L</slurpy> method against a type constraint.  For example:
 
     use MooseX::Types::Structured qw(Tuple slurpy);
-    
+
     subtype AllowTailingArgs,
      as Tuple[
        Int,
@@ -568,7 +580,7 @@ use the L</slurpy> method against a type constraint.  For example:
 This will now work as expected, validating ArrayRef structures such as:
 
     [1,"hello", $obj, 2,3,4,5,6,...]
-    
+
 A few caveats apply.  First, the slurpy type constraint must be the last one in
 the list of type constraint parameters.  Second, the parent type of the slurpy
 type constraint must match that of the containing type constraint.  That means
@@ -612,25 +624,25 @@ constraint and coercions.  This example also shows structured types mixed which
 other MooseX::Types libraries.
 
     package Test::MooseX::Meta::TypeConstraint::Structured::Examples::Normalize;
-    
+
     use Moose;
     use DateTime;
-    
+
     use MooseX::Types::Structured qw(Dict Tuple);
     use MooseX::Types::DateTime qw(DateTime);
     use MooseX::Types::Moose qw(Int Str Object);
     use MooseX::Types -declare => [qw(Name Age Person)];
-     
+
     subtype Person,
      as Dict[
      	name=>Str,
      	age=>Int,
      ];
-    
+
     coerce Person,
      from Dict[
-     	first=>Str, 
-     	last=>Str, 
+     	first=>Str,
+     	last=>Str,
      	years=>Int,
      ], via { +{
         name => "$_->{first} $_->{last}",
@@ -638,9 +650,9 @@ other MooseX::Types libraries.
      }},
      from Dict[
      	fullname=>Dict[
-     		last=>Str, 
+     		last=>Str,
      		first=>Str,
-     	], 
+     	],
      	dob=>DateTime,
      ],
      ## DateTime needs to be inside of single quotes here to disambiguate the
@@ -650,18 +662,18 @@ other MooseX::Types libraries.
         name => "$_->{fullname}{first} $_->{fullname}{last}",
         age => ($_->{dob} - 'DateTime'->now)->years,
      }};
-     
+
     has person => (is=>'rw', isa=>Person, coerce=>1);
-    
+
 And now you can instantiate with all the following:
 
     __PACKAGE__->new(
         person=>{
             name=>'John Napiorkowski',
-            age=>39,            
+            age=>39,
         },
     );
-        
+
     __PACKAGE__->new(
         person=>{
             first=>'John',
@@ -669,7 +681,7 @@ And now you can instantiate with all the following:
             years=>39,
         },
     );
-    
+
     __PACKAGE__->new(
         person=>{
             fullname => {
@@ -680,31 +692,57 @@ And now you can instantiate with all the following:
                 year=>1969,
                 month=>2,
                 day=>13
-            ),            
+            ),
         },
     );
-    
+
 This technique is a way to support various ways to instantiate your class in a
 clean and declarative way.
 
 =cut
 
+my $Optional = MooseX::Meta::TypeConstraint::Structured::Optional->new(
+    name => 'MooseX::Types::Structured::Optional',
+    package_defined_in => __PACKAGE__,
+    parent => find_type_constraint('Item'),
+    constraint => sub { 1 },
+    constraint_generator => sub {
+        my ($type_parameter, @args) = @_;
+        my $check = $type_parameter->_compiled_type_constraint();
+        return sub {
+            my (@args) = @_;
+            ## Does the arg exist?  Something exists if it's a 'real' value
+            ## or if it is set to undef.
+            if(exists($args[0])) {
+                ## If it exists, we need to validate it
+                $check->($args[0]);
+            } else {
+                ## But it's is okay if the value doesn't exists
+                return 1;
+            }
+        }
+    }
+);
+
+Moose::Util::TypeConstraints::register_type_constraint($Optional);
+Moose::Util::TypeConstraints::add_parameterizable_type($Optional);
+
 Moose::Util::TypeConstraints::get_type_constraint_registry->add_type_constraint(
 	MooseX::Meta::TypeConstraint::Structured->new(
 		name => "MooseX::Types::Structured::Tuple" ,
 		parent => find_type_constraint('ArrayRef'),
-		constraint_generator=> sub { 
+		constraint_generator=> sub {
 			## Get the constraints and values to check
             my ($type_constraints, $values) = @_;
 			my @type_constraints = defined $type_constraints ?
              @$type_constraints : ();
-            
+
             my $overflow_handler;
             if($type_constraints[-1] && blessed $type_constraints[-1]
               && $type_constraints[-1]->isa('MooseX::Types::Structured::OverflowHandler')) {
                 $overflow_handler = pop @type_constraints;
             }
-            
+
 			my @values = defined $values ? @$values: ();
 			## Perform the checking
 			while(@type_constraints) {
@@ -715,10 +753,10 @@ Moose::Util::TypeConstraints::get_type_constraint_registry->add_type_constraint(
                         $_[2]->{message} = $type_constraint->get_message($value)
                          if ref $_[2];
 						return;
-					}				
+					}
 				} else {
                     ## Test if the TC supports null values
-					unless($type_constraint->check()) {
+                    unless ($type_constraint->is_subtype_of($Optional)) {
                         $_[2]->{message} = $type_constraint->get_message('NULL')
                          if ref $_[2];
 						return;
@@ -745,22 +783,22 @@ Moose::Util::TypeConstraints::get_type_constraint_registry->add_type_constraint(
 		}
 	)
 );
-	
+
 Moose::Util::TypeConstraints::get_type_constraint_registry->add_type_constraint(
 	MooseX::Meta::TypeConstraint::Structured->new(
 		name => "MooseX::Types::Structured::Dict",
 		parent => find_type_constraint('HashRef'),
-		constraint_generator=> sub { 
+		constraint_generator=> sub {
 			## Get the constraints and values to check
             my ($type_constraints, $values) = @_;
 			my @type_constraints = defined $type_constraints ?
              @$type_constraints : ();
-            
+
             my $overflow_handler;
             if($type_constraints[-1] && blessed $type_constraints[-1]
               && $type_constraints[-1]->isa('MooseX::Types::Structured::OverflowHandler')) {
                 $overflow_handler = pop @type_constraints;
-            } 
+            }
             my (%type_constraints) = @type_constraints;
 			my %values = defined $values ? %$values: ();
 			## Perform the checking
@@ -777,7 +815,7 @@ Moose::Util::TypeConstraints::get_type_constraint_registry->add_type_constraint(
 					}
 				} else {
                     ## Test to see if the TC supports null values
-					unless($type_constraint->check()) {
+                    unless ($type_constraint->is_subtype_of($Optional)) {
                         $_[2]->{message} = $type_constraint->get_message('NULL')
                          if ref $_[2];
 						return;
@@ -785,7 +823,7 @@ Moose::Util::TypeConstraints::get_type_constraint_registry->add_type_constraint(
 				}
 			}
 			## Make sure there are no leftovers.
-			if(%values) { 
+			if(%values) {
                 if($overflow_handler) {
                     return $overflow_handler->check(+{%values});
                 } else {
@@ -805,33 +843,45 @@ Moose::Util::TypeConstraints::get_type_constraint_registry->add_type_constraint(
 	)
 );
 
-OPTIONAL: {
-    my $Optional = Moose::Meta::TypeConstraint::Parameterizable->new(
-        name => 'MooseX::Types::Structured::Optional',
-        package_defined_in => __PACKAGE__,
-        parent => find_type_constraint('Item'),
-        constraint => sub { 1 },
-        constraint_generator => sub {
-            my ($type_parameter, @args) = @_;
-            my $check = $type_parameter->_compiled_type_constraint();
-            return sub {
-                my (@args) = @_;
-                ## Does the arg exist?  Something exists if it's a 'real' value
-                ## or if it is set to undef.
-                if(exists($args[0])) {
-                    ## If it exists, we need to validate it
-                    $check->($args[0]);
-                } else {
-                    ## But it's is okay if the value doesn't exists
-                    return 1;
-                }
-            }
-        }
-    );
+Moose::Util::TypeConstraints::get_type_constraint_registry->add_type_constraint(
+  MooseX::Meta::TypeConstraint::Structured->new(
+    name => "MooseX::Types::Structured::Map",
+    parent => find_type_constraint('HashRef'),
+    constraint_generator=> sub {
+      ## Get the constraints and values to check
+      my ($type_constraints, $values) = @_;
+      my @constraints = defined $type_constraints ? @$type_constraints : ();
 
-    Moose::Util::TypeConstraints::register_type_constraint($Optional);
-    Moose::Util::TypeConstraints::add_parameterizable_type($Optional);
-}
+      Carp::confess( "too many args for Map type" ) if @constraints > 2;
+
+      my ($key_type, $value_type) = @constraints == 2 ? @constraints
+                                  : @constraints == 1 ? (undef, @constraints)
+                                  :                     ();
+
+      my %values = defined $values ? %$values: ();
+      ## Perform the checking
+      if ($value_type) {
+        for my $value (values %$values) {
+          unless ($value_type->check($value)) {
+            $_[2]->{message} = $value_type->get_message($value) if ref $_[2];
+            return;
+          }
+        }
+      }
+
+      if ($key_type) {
+        for my $key (keys %$values) {
+          unless ($key_type->check($key)) {
+            $_[2]->{message} = $key_type->get_message($key) if ref $_[2];
+            return;
+          }
+        }
+      }
+
+      return 1;
+    },
+  )
+);
 
 sub slurpy ($) {
 	my ($tc) = @_;
@@ -854,7 +904,7 @@ Here's a list of stuff I would be happy to get volunteers helping with:
 	* All POD examples need test cases in t/documentation/*.t
 	* Want to break out the examples section to a separate cookbook style POD.
 	* Want more examples and best practice / usage guidance for authors
-	* Need to clarify deep coercions, 
+	* Need to clarify deep coercions,
 
 =head1 AUTHOR
 
@@ -877,5 +927,5 @@ This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 =cut
-	
+
 1;
